@@ -1,39 +1,41 @@
-import unittest
-import logging
-import requests
 import json
 import os
-import datetime
+import random
+import string
 import time
-import socket
-import s3fs
+import unittest
+import uuid
+import jwt
+from unittest.mock import patch
 
-from six.moves.urllib import parse
-from mock import patch
-from dli.client import session, builders
+import requests
+from urllib import parse
 
+from dli.client import builders, session
+from tests import localstack_helper
 
-logger = logging.getLogger(__name__)
-# this token allows us to login when running datacat in dev mode
-token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiIsImlhdCI6MTUxMzMzMTUwOSwiZXhwIjo5NTEzMzM1MTA5fQ.eyJhdWQiOiJkYXRhbGFrZS1hY2NvdW50cyIsImF1dGhfdGltZSI6MTUxMzMzMTUwOSwiZGF0YWxha2UiOnsiYWNjb3VudHMiOnsiaWJveHgiOiJydyIsIm1yZCI6InIiLCJkYXRhbGFrZS1tZ210IjoicncifX0sImVtYWlsIjoiamltQGV4YW1wbGUuY29tIiwiZW1haWxfdmVyaWZpZWQiOnRydWUsImV4cCI6OTUxMzMzNTEwOSwiaWF0IjoxNTEzMzMxNTA5LCJpc3MiOiJodHRwczovL2h5ZHJhLWRldi51ZHBtYXJraXQubmV0Iiwibm9uY2UiOiI1YTgyMzI2NC1iN2QzLTQ4NWUtYjc2MS1lYzE1MTI5NDQ4MTQiLCJzdWIiOiJ1c2VyOjEyMzQ1OmppbSJ9.mB9-W8KCyARq5zkLsjSdmE4KKQ7Q7MBvmX5J6rYMMtM"
+token_data = {
+  "aud": "datalake-catalogue-dev",
+  "auth_time": 1513331509,
+  "datalake": {
+    "accounts": {
+      "iboxx": "rw",
+      "mrd": "r",
+      "datalake-mgmt": "rw"
+    },
+    "tenant_id": "4e27686c-e72a-44f1-a126-f0d2940617d4",
+    "organisation_id": "4e27686c-e72a-44f1-a126-f0d2940617d4",
+  },
+  "email": "jim@example.com",
+  "email_verified": True,
+  "exp": 9513335109,
+  "iat": 1513331509,
+  "iss": "https://hydra-dev.udpmarkit.net",
+  "nonce": "5a823264-b7d3-485e-b761-ec1512944814",
+  "sub": "user:12345:jim"
+}
 
-
-def build_fake_s3fs(key, secret, token):
-    # Nasty: Resolve the hostname so that the we connect
-    # using the ip address instead of the hostname
-    # this is due a limitation in fake-s3, read more here:
-    # https://github.com/jubos/fake-s3/issues/17
-    s3_hostname = socket.gethostbyname(os.environ.get("FAKE_S3_HOST", "localhost"))
-    s3_port = os.environ.get("FAKE_S3_PORT", 4569)
-
-    return s3fs.S3FileSystem(
-        key=key,
-        secret=secret,
-        token=token,
-        client_kwargs={
-            "endpoint_url": "http://%s:%s" % (s3_hostname, s3_port)
-        }
-    )
+token = jwt.encode(token_data, 'secret').decode('utf')
 
 
 class SdkIntegrationTestCase(unittest.TestCase):
@@ -49,22 +51,6 @@ class SdkIntegrationTestCase(unittest.TestCase):
     # By now we patch the response to return a token we know will authenticate
     # instead of generating an api key, this isn't ideal as the behaviour is
     # not exactly the same, but it is good enough by now.
-    @patch("dli.client.dli_client._get_auth_key", lambda k, r, h: token)
-    def setUp(self):
-        self.headers = {
-            'Content-type': 'application/json',
-            'Authorization': 'Bearer {}'.format(token),
-            'Cookie': 'oidc_id_token=' + token
-        }
-        self.root_url = os.environ.get("DATA_LAKE_INTERFACE_URL", "http://localhost:9000/__api/")
-        self.api_key = self.get_api_key()
-        self.client = self.create_session()
-        self.s3 = []
-
-        # create a fake s3 repo
-        self.set_s3_client_mock()
-        self.register_aws_account(self.aws_account_id)
-
     def set_s3_client_mock(self):
         def _upload(_, files, location, tr=None, r=None):
             result = []
@@ -76,13 +62,32 @@ class SdkIntegrationTestCase(unittest.TestCase):
 
         s3_upload = patch('dli.client.s3.Client.upload_files_to_s3', _upload)
         s3_upload.start()
-        s3_download = patch('dli.client.s3.Client.download_file')
+        s3_download = patch('dli.client.s3.Client.download_files_from_s3_path')
         self.s3_download_mock = s3_download.start()
 
         # cleanup
         self.addCleanup(s3_upload.stop)
         self.addCleanup(s3_download.stop)
 
+    @patch("dli.client.dli_client.Session._get_auth_key", lambda self: token)
+    @patch("dli.client.dli_client.AnalyticsHandler", lambda self: None)
+    def setUp(self):
+        self.headers = {
+            'Content-type': 'application/json',
+            'Authorization': 'Bearer {}'.format(token),
+            'Cookie': 'oidc_id_token=' + token
+        }
+        self.root_url = os.environ.get(
+            "DATA_LAKE_INTERFACE_URL", "http://data-lake-interface:8080/__api/")
+        self.api_key = self.get_api_key()
+        self.client = self.create_session()
+        self.s3 = []
+
+        # create a fake s3 repo
+        self.set_s3_client_mock()
+        self.register_aws_account(self.aws_account_id)
+        self.s3_client = localstack_helper.get_s3_client()
+
     def get_api_key(self):
         # url = "%s/generate-my-key" % self.root_url
         # payload = {
@@ -97,7 +102,7 @@ class SdkIntegrationTestCase(unittest.TestCase):
 
     def create_package(self, name, access="Restricted", **kwargs):
         payload = {
-            "name": name + "_" + str(datetime.datetime.now()),
+            "name": name + "_" + str(uuid.uuid4()),
             "description":  "asd",
             "publisher":  "datalake-mgmt",
             "manager_id":  "datalake-mgmt",
@@ -112,25 +117,28 @@ class SdkIntegrationTestCase(unittest.TestCase):
         payload.update(**kwargs)
         return self.client.register_package(**payload).package_id
 
-    def create_collection(self, name):
-        return self.client.create_collection(
-            name=name + "_" + str(datetime.datetime.now()),
-            description="test",
-            manager_id="datalake-mgmt",
-        ).collection_id
-
-    def dataset_builder(self, package_id, dataset_name):
+    def dataset_builder(self, package_id, dataset_name, **kwargs):
+        arguments = dict(kwargs)
+        data_preview_type = arguments.pop('data_preview_type', 'NONE')
+        default_short_code = ''.join(random.choices(
+            string.ascii_letters + string.digits, k=10
+        ))
+        short_code = arguments.pop('short_code', default_short_code)
         return builders.DatasetBuilder(
             package_id=package_id,
             name=dataset_name,
-            description="a testing dataset",
-            content_type="Pricing",
-            data_format="CSV",
-            publishing_frequency="Daily",
-            taxonomy=[]
+            description='a testing dataset',
+            content_type='Pricing',
+            data_format='CSV',
+            publishing_frequency='Daily',
+            taxonomy=[],
+            data_preview_type=data_preview_type,
+            short_code=short_code,
+            **arguments
         )
 
-    def register_s3_dataset(self, package_id, dataset_name, bucket_name, prefix="/"):
+    def register_s3_dataset(self, package_id, dataset_name, bucket_name, prefix="prefix"):
+        self._setup_bucket_and_prefix(bucket_name, prefix)
         return self.client.register_dataset(
             self.dataset_builder(package_id, dataset_name).with_external_s3_storage(
                 bucket_name,
@@ -139,6 +147,26 @@ class SdkIntegrationTestCase(unittest.TestCase):
             )
         )
 
+    def _setup_bucket_and_prefix(self, bucket_name, prefix):
+        # API checks whether DataLake can access S3 bucket at prefix. If dataset is registered with S3 location
+        # we must create a fake one in localstack.
+        s3_client = self.s3_client
+        s3_client.create_bucket(Bucket=bucket_name)
+        s3_client.put_object(
+            Bucket=bucket_name,
+            # Ensure key ends in a slash to signify it's a directory
+            Key=prefix.rstrip('/') + '/',
+            Body=""
+        )
+
+        def cleanup_bucket():
+            for object_to_delete in s3_client.list_objects(Bucket=bucket_name)['Contents']:
+                s3_client.delete_objects(Bucket=bucket_name, Delete={"Objects": [{"Key": object_to_delete['Key']}]})
+            s3_client.delete_bucket(Bucket=bucket_name)
+
+        self.addCleanup(cleanup_bucket)
+
+
     def create_session(self):
         return session.start_session(self.api_key, self.root_url)
 
@@ -149,10 +177,11 @@ class SdkIntegrationTestCase(unittest.TestCase):
             data=json.dumps({
                 "awsAccountId": str(aws_account_id),
                 "awsAccountName": str(aws_account_id),
-                "awsAccessKeyId": "accessKey",
-                "awsSecretAccessKey": "secretAccessKey",
+                "awsAccessKeyId": None,
+                "awsSecretAccessKey": None,
                 "awsRegion": "eu-west-1",
-                "accountIds": ["datalake-mgmt"]
+                "accountIds": ["datalake-mgmt"],
+                "awsRoleArn": 'arn:aws:iam::000000000001:user/root'
             })
         )
 
